
template< typename T >
void
curlpp::registerer< T >::record_registry( curlpp::registry_base< T > *registry )
{
   typename std::list< curlpp::registry_base< T > * >::iterator registry_pos =
      m_registry_list.begin();
   while( 
         ( registry_pos != m_registry_list.end() ) && 
         ( (*registry_pos ) !=  registry ) )
   {
      registry_pos++;
   }

   if( registry_pos == m_registry_list.end() )
   {
      m_registry_list.push_back( registry );
      for(
            typename std::list< T >::iterator handle_pos = m_handles_list.begin();
            handle_pos != m_handles_list.end();
            handle_pos++
         )
      {
         registry->record( (*handle_pos) );
      }
   }
}

template< typename T >
void 
curlpp::registerer< T >::unrecord_registry
( 
      curlpp::registry_base< T > *registry,
      bool unregister_handles 
)
{
   typename std::list< curlpp::registry_base< T > * >::iterator pos =
      m_registry_list.begin();
   while( ( pos != m_registry_list.end() ) && ( (*pos) != registry ) )
   {
      pos++;
   }

   if( pos != m_registry_list.end() )
   {
      m_registry_list.erase( pos );
      if( unregister_handles )
      {
         for(
               typename std::list< T >::iterator handle_pos = m_handles_list.begin();
               handle_pos != m_handles_list.end();
               handle_pos++
            )
         {
            registry->unrecord( (*handle_pos) );
         }
      }
   }
}

template< typename T >
void
curlpp::registerer< T >::record( const T &handle )
{
   for(
         typename std::list< curlpp::registry_base< T > * >::iterator pos =
         m_registry_list.begin();
         pos != m_registry_list.end();
         pos++
      )
   {
      (*pos)->record( handle );
   }
}

template< typename T >
void
curlpp::registerer< T >::unrecord( const T &handle )
{
   typename std::list< T >::iterator handle_pos = m_handles_list.begin();
   while( 
         handle_pos != m_handles_list.end() &&
         (*handle_pos) != handle
        )
   {
      handle_pos++;
   }

   if( handle_pos != m_handles_list.end() )
   {
      typename std::list< curlpp::registry_base< T > * >::iterator pos;
      for(
            pos = m_registry_list.begin();
            pos != m_registry_list.end();
            pos++
         )
      {
         (*pos)->unrecord( handle );
      }

      m_handles_list.erase( handle_pos );
   }
}

template< typename T >
curlpp::registerer< T >::~registerer()
{}
