#include "lifetime_library.hpp"

utilspp::lifetime_library_impl::lifetime_library_impl()
   :
   m_tracker_array( NULL ),
   m_nb_elements( 0 )
{}

utilspp::lifetime_library_impl::~lifetime_library_impl()
{
   terminate();
}

void
utilspp::lifetime_library_impl::add( utilspp::private_members::lifetime_tracker *tracker )
{
   utilspp::private_members::tracker_array new_array = static_cast<
      utilspp::private_members::tracker_array >( 
         std::realloc( 
            m_tracker_array, 
            m_nb_elements + 1
            )
         );
   if( new_array == NULL )
   {
      throw std::bad_alloc();
   }

   m_tracker_array = new_array;

   utilspp::private_members::tracker_array pos = std::upper_bound( 
         m_tracker_array, 
         m_tracker_array + m_nb_elements,
         tracker,
         &utilspp::private_members::lifetime_tracker::compare
         );
   std::copy_backward( 
         pos, 
         m_tracker_array + m_nb_elements, 
         m_tracker_array + m_nb_elements + 1
         );

   *pos = tracker;
   m_nb_elements++;
};

void
utilspp::lifetime_library_impl::terminate()
{
   //The number of elements MUST always be equal or over zero.
   assert( m_nb_elements >= 0 );

   while( m_nb_elements > 0 )
   {
      //At this point the m_tracker_array MUST not be NULL.
      assert( m_tracker_array != NULL );
      
      //Pick the element at the top of the stack.
      utilspp::private_members::lifetime_tracker* top =  
         m_tracker_array[ 
         m_nb_elements - 1
         ];

      //Remove that object off the stack.
      //Don't check errors-realloc with less memory, cause that can't fail.
      m_tracker_array = 
         static_cast< utilspp::private_members::tracker_array >
         (
               std::realloc( 
                  m_tracker_array, 
                  --m_nb_elements
                  )
         );

      //Destroy the element.
      delete top;
   }
}

unsigned int 
utilspp::get_longevity( utilspp::lifetime_library_impl * )
{
   return 0;
}

