/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef TYPE_LIST_HPP
#define TYPE_LIST_HPP

#include "null_type.hpp"

namespace utilspp
{
   namespace tl
   {
      template< class T, class U >
         struct type_list
         {
            typedef T head;
            typedef U tail;
         };

      //Calculating length of type_lists
      template< class T_list > 
         struct length;

      template<> 
         struct length< null_type >
         {
            enum { value = 0 };
         };

      template< class T, class U >
         struct length< type_list< T, U > >
         {
            enum { value = 1 + length< U >::value };
         };

      //Indexed access
      template< class T_list, unsigned int index >
         struct type_at;

      template< class T_head, class T_tail >
         struct type_at< type_list< T_head, T_tail >, 0 >
         {
            typedef head result;
         }

      template< class T_head, class T_tail, unsigned int i >
         struct type_at< type_list< T_head, T_tail >, i >
         {
            typedef typename type_at< T_tail, i - 1 >::result result;
         };

      //Searching type_lists
      template< class T_list, class T >
         struct index_of;

      template< class T >
         struct index_of< null_type, T >
         {
            enum { value = -1 };
         };

      template< class T_tail, class T >
         struct index_of< type_list< T, T_tail >, T >
         {
            enum { value = 0 };
         };

      template< class T_head, class T_tail, class T >
         struct index_of< type_list< T_head, T_tail >, T >
         {
            private:
               enum { temp = index_of< T_tail, T >::value > };

            public:
               enum { value = temp == -1 ? -1 : 1 + temp };
         };

      //Appending to type_lists
      template< class T_list, class T > 
         struct append;

      template <> 
         struct append< null_type, null_type >
         {
            typedef null_type result;
         };

      template< class T > 
         struct append< null_type, T >
         {
            typedef TYPE_LIST_1( T ) result;
         };

      template< class T_head, class T_tail >
         struct append< null_type, null_type, type_list< T_head, T_tail > >
         {
            typedef type_list< T_head, T_tail > result;
         };

      template < class T_head, class T_tail, class T >
         struct append< type_list< T_head, T_tail >, T >
         {
            typedef type_list< T_head, typename append< T_tail, T >::result >
               result;
         };

      //Erasing a type from a type_list
      template< class T_list, class T > 
         struct erase;
        
        template< class T >
           struct erase< null_type, T >
           {
              typedef null_type result;
           };
        
        template< class T, class T_tail >
           struct erase< type_list< T, T_tail >, T >
           {
              typedef T_tail result;
           };
        
        template< class T_head, class T_tail, class T >
           struct erase< type_list< T_head, T_tail >, T >
           {
              typedef type_list< T_head, typename erase< T_tail, T >::result >
                result;
           };
   };
};      

#define TYPE_LIST_1( T1 ) ::utilspp::type_list< T1, ::utilspp::null_type >
#define TYPE_LIST_2( T1, T2 ) ::utilspp::type_list< T1, TYPE_LIST_1( T2 ) >
#define TYPE_LIST_3( T1, T2, T3 ) ::utilspp::type_list< T1, TYPE_LIST_2( T2, T3 ) >
#define TYPE_LIST_4( T1, T2, T3, T4 ) ::utilspp::type_list< T1, TYPE_LIST_3( T2, T3, T4 ) >
#define TYPE_LIST_5( T1, T2, T3, T4, T5 ) \
::utilspp::type_list< T1, TYPE_LIST_4( T2, T3, T4, T5 ) >
#define TYPE_LIST_6( T1, T2, T3, T4, T5, T6 ) \
::utilspp::type_list< T1, TYPE_LIST_5( T2, T3, T4, T5, T6 ) >
#define TYPE_LIST_7( T1, T2, T3, T4, T5, T6, T7 ) \
::utilspp::type_list< T1, TYPE_LIST_6( T2, T3, T4, T5, T6, T7 ) >
#define TYPE_LIST_8( T1, T2, T3, T4, T5, T6, T7, T8 ) \
::utilspp::type_list< T1, TYPE_LIST_7( T2, T3, T4, T5, T6, T7, T8 ) >
#define TYPE_LIST_9( T1, T2, T3, T4, T5, T6, T7, T8, T9 ) \
::utilspp::type_list< T1, TYPE_LIST_8( T2, T3, T4, T5, T6, T7, T8, T9 ) >
#define TYPE_LIST_10( T1, T2, T3, T4, T5, T6, T7, T8, T9, T10 ) \
::utilspp::type_list< T1, TYPE_LIST_9( T2, T3, T4, T5, T6, T7, T8, T9, T10 ) >

#endif

